/**********************************************************************
 *
 * This code is part of the OOML Probot project
 * Authors: Alberto Valer-Gomez, Nieves Cubo Mateo, Mario Almagro
 *
 * OOML is licenced under the Common Creative License,
 * Attribution-ShareAlike 3.0
 *
 * You are free:
 *   - to Share - to copy, distribute and transmit the work
 *   - to Remix - to adapt the work
 *
 * Under the following conditions:
 *   - Attribution. You must attribute the work in the manner specified
 *     by the author or licensor (but not in any way that suggests that
 *     they endorse you or your use of the work).
 *   - Share Alike. If you alter, transform, or build upon this work,
 *     you may distribute the resulting work only under the same or
 *     similar license to this one.
 *
 * Any of the above conditions can be waived if you get permission
 * from the copyright holder.  Nothing in this license impairs or
 * restricts the author's moral rights.
 *
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 **********************************************************************/

#include "front_usir.h"
#include <components.h>
#include <core.h>
#include <parts/ArduinoUNO.h>

Component FrontUSIR::build()
{
   //MAIN BODY TO HOLD THE SENSORS
   Component body = RoundedTablet(10,20,_bthickness,5,true,true,false,false).translate(8.75,18,1.25)
           + RoundedTablet(10,20,2.5,5,true,true,false,false).translate(-19.25,18,1.25)
           ;

    Polygon<Point2D> triangle;
    triangle.addPoint(Point2D(104.5/2,0));
    triangle.addPoint(Point2D(-104.5/2,0));
    triangle.addPoint(Point2D(0,-104.5/2*sin(deg2rad(35))));
    Component body_head = PolygonalPrism(triangle,_bthickness);
    body_head.translate(0,9.5,0);//.color(1,0,0);

    Component front_body = body_head * RoundedTablet(107.75,20,_bthickness,5,false,false)
            .translate(0,0,_bthickness/2);

    body = body + front_body;

    Component arduino = ArduinoUNO::getBoard(2,20,true).rotate(0,0,-90).translate(-26.75,80,5);


    double sx,sy,thickness;

    sx = SeeedUSSensor().getData().sx;
    sy = SeeedUSSensor().getData().sy;
    thickness = SeeedUSSensor().getData().board_thickness;


    Component ussupport1 = RoundedTablet(1.2*sx,1.3*sy,_vthickness, 6, true, true , false, false)
            + RoundedTablet(1.2*sx,6,2.5,1).rotate(90,0,0).translate(0,-1.3*sy/2+1.25,0)
               - SeeedUSSensor().translate(0,0,-thickness -_vthickness/2);
    Component ussupport2 = RoundedTablet(1.2*sx,1.3*sy,_vthickness, 6, true, true , false, false)
            + RoundedTablet(1.2*sx,6,2.5,1).rotate(90,0,0).translate(0,-1.3*sy/2+1.25,0)
               - SeeedUSSensor().translate(0,0,-thickness - _vthickness/2);

    ussupport1.rotate(0,30,0);
    ussupport2.rotate(0,-30,0);

    double x_trans = 1.3*sx/2 * cos (deg2rad(30));

    ussupport1.translate(x_trans-0.65,0,0);
    ussupport2.translate(-x_trans+0.65,0,0);

    Component us2support = ussupport1.translate(18,0,0)
               + ussupport2.translate(-18,0,0);
    us2support.rotate(90,0,0).translate(0,-5,1.3*sy/2);
    us2support.translate(0,12,0);

    double irsx = SharpIRSensor().getData().sx;
    double irsy = SharpIRSensor().getData().sy;

    Component irsupport = RoundedTablet(1.1*irsy,1.1*irsx,_vthickness,6,true,true,false,false)
            + Cube::create(_vthickness,0.9*irsx,_vthickness).translate(1.1*irsy/2 - _vthickness/2 , -(1.1-0.9)*irsx/2 ,-_vthickness)
            + Cube::create(_vthickness,0.9*irsx,_vthickness).translate(-1.1*irsy/2 + _vthickness/2 , -(1.1-0.9)*irsx/2 ,-_vthickness)
            ;

    irsupport.rotate(90,0,0).translate(0,6,1.1*irsx/2);


    Component ir = SharpIRSensor(true);
    ir.rotate(90,90,0).translate(0,4,25);


    irsupport = irsupport - ir;
    irsupport.translate(0,-10,0);


    Component ir2usplugin = body + us2support;
    ir2usplugin.translate(0,-2.5,0);
    ir2usplugin = ir2usplugin + irsupport;

    ir2usplugin = ir2usplugin.translate(0,-7,0) - arduino;

    return ir2usplugin;

}

Component FrontUSIR::getIRSensor() const {
    Component ir = SharpIRSensor(false);
    ir.rotate(90,90,0).translate(0,-15.5,25);
    return ir;
}

Component FrontUSIR::getLeftUSSensor() const {

    double sx,sy,thickness;

    sx = SeeedUSSensor().getData().sx;
    sy = SeeedUSSensor().getData().sy;
    thickness = SeeedUSSensor().getData().board_thickness;


    Component us = SeeedUSSensor(false).translate(0,0,-thickness-1.75);

    us.rotate(0,30,0);

    double x_trans = 1.2*sx/2 * cos (deg2rad(30));

    us.translate(x_trans-0.65,0,0);

    us.translate(18,0,0);

    us.rotate(90,0,0).translate(0,-5,1.3*sy/2);
    us.translate(0,10,0);
    us.translate(0,-7.5,0);

    return us;
}

Component FrontUSIR::getRightUSSensor() const {

    double sx,sy,thickness;

    sx = SeeedUSSensor().getData().sx;
    sy = SeeedUSSensor().getData().sy;
    thickness = SeeedUSSensor().getData().board_thickness;

    Component us = SeeedUSSensor(false).translate(0,0,-thickness-1.75);

    us.rotate(0,-30,0);

    double x_trans = 1.2*sx/2 * cos (deg2rad(30));

    us.translate(-x_trans+0.65,0,0);

    us.translate(-18,0,0);
    us.rotate(90,0,0).translate(0,-5,1.3*sy/2);
    us.translate(0,10,0);
    us.translate(0,-7.5,0);

    return us;
}
