/**********************************************************************
 *
 * This code is part of the OOML Probot project
 * Authors: Alberto Valer-Gomez, Nieves Cubo Mateo, Mario Almagro
 *
 * OOML is licenced under the Common Creative License,
 * Attribution-ShareAlike 3.0
 *
 * You are free:
 *   - to Share - to copy, distribute and transmit the work
 *   - to Remix - to adapt the work
 *
 * Under the following conditions:
 *   - Attribution. You must attribute the work in the manner specified
 *     by the author or licensor (but not in any way that suggests that
 *     they endorse you or your use of the work).
 *   - Share Alike. If you alter, transform, or build upon this work,
 *     you may distribute the resulting work only under the same or
 *     similar license to this one.
 *
 * Any of the above conditions can be waived if you get permission
 * from the copyright holder.  Nothing in this license impairs or
 * restricts the author's moral rights.
 *
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 **********************************************************************/

#ifndef FRONTUSIR_INCLUDED
#define FRONTUSIR_INCLUDED

#ifdef WIN32
#ifdef OOMLParts_EXPORTS
#define OOMLParts_EXP_DEC __declspec(dllexport)
#else
#define OOMLParts_EXP_DEC __declspec(dllimport)
#endif
#else
#define OOMLParts_EXP_DEC
#endif

#include <core/AbstractPart.h>
#include <parts/SeeedUSSensor.h>
#include <parts/SharpIRSensor.h>

/**
 * \brief Front US, IR sesnor plugin for Protobot
 *
 * This class provides a Front US, IR sesnor plugin for Protobot
 */
class OOMLParts_EXP_DEC FrontUSIR : public AbstractPart
{
public:
    /**
     * \brief Default constructor.
     */
    FrontUSIR() :
        AbstractPart(),
        _bthickness(2.5),
        _vthickness(3.5)
    {
        rebuild();
    }
    /**
     * \brief Default parametrized constructor.
     *
     * \param vertical_thickness thickness of vertical walls
     * \param vertical_thickness thickness base
     */
    FrontUSIR(double vertical_thickness, double base_thickness=2.5) :
        AbstractPart(),
        _vthickness(vertical_thickness),
        _bthickness(base_thickness)
    {
        rebuild();
    }
    /**
     * \brief Default destructor.
     */
    virtual ~FrontUSIR() {}

    /**
      * returns the model of the Left UltraSound Sensor
      */
    Component getLeftUSSensor() const;
    /**
      * return the model of the Right UltraSound Sensor on its proper position
      */
    Component getRightUSSensor() const;

    /**
      * return the model of the IR Sensor  on its proper position
      */
    Component getIRSensor() const;

protected:
    /**
     * \brief Build the piece.
     *
     * This method build the piece from simpler objects.
     *
     * \return The piece built.
     */
    virtual Component build();

private:
    double _vthickness; /** Thickness of vertical walls */
    double _bthickness; /** Thickness of base */
};

#endif
